# This file is generated by objective.metadata
#
# Last update: Sun Feb 20 18:30:45 2022
#
# flake8: noqa

import objc, sys
from typing import NewType

if sys.maxsize > 2**32:

    def sel32or64(a, b):
        return b

else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a

else:

    def selAorI(a, b):
        return b


misc = {}
constants = """$kCFDNSServiceFailureKey$kCFErrorDomainCFNetwork$kCFErrorDomainWinSock$kCFFTPResourceGroup$kCFFTPResourceLink$kCFFTPResourceModDate$kCFFTPResourceMode$kCFFTPResourceName$kCFFTPResourceOwner$kCFFTPResourceSize$kCFFTPResourceType$kCFFTPStatusCodeKey$kCFGetAddrInfoFailureKey$kCFHTTPAuthenticationAccountDomain$kCFHTTPAuthenticationPassword$kCFHTTPAuthenticationSchemeBasic$kCFHTTPAuthenticationSchemeDigest$kCFHTTPAuthenticationSchemeKerberos$kCFHTTPAuthenticationSchemeNTLM$kCFHTTPAuthenticationSchemeNegotiate$kCFHTTPAuthenticationSchemeNegotiate2$kCFHTTPAuthenticationSchemeOAuth1$kCFHTTPAuthenticationSchemeXMobileMeAuthToken$kCFHTTPAuthenticationUsername$kCFHTTPVersion1_0$kCFHTTPVersion1_1$kCFHTTPVersion2_0$kCFHTTPVersion3_0$kCFNetworkProxiesExceptionsList$kCFNetworkProxiesExcludeSimpleHostnames$kCFNetworkProxiesFTPEnable$kCFNetworkProxiesFTPPassive$kCFNetworkProxiesFTPPort$kCFNetworkProxiesFTPProxy$kCFNetworkProxiesGopherEnable$kCFNetworkProxiesGopherPort$kCFNetworkProxiesGopherProxy$kCFNetworkProxiesHTTPEnable$kCFNetworkProxiesHTTPPort$kCFNetworkProxiesHTTPProxy$kCFNetworkProxiesHTTPSEnable$kCFNetworkProxiesHTTPSPort$kCFNetworkProxiesHTTPSProxy$kCFNetworkProxiesProxyAutoConfigEnable$kCFNetworkProxiesProxyAutoConfigJavaScript$kCFNetworkProxiesProxyAutoConfigURLString$kCFNetworkProxiesProxyAutoDiscoveryEnable$kCFNetworkProxiesRTSPEnable$kCFNetworkProxiesRTSPPort$kCFNetworkProxiesRTSPProxy$kCFNetworkProxiesSOCKSEnable$kCFNetworkProxiesSOCKSPort$kCFNetworkProxiesSOCKSProxy$kCFProxyAutoConfigurationHTTPResponseKey$kCFProxyAutoConfigurationJavaScriptKey$kCFProxyAutoConfigurationURLKey$kCFProxyHostNameKey$kCFProxyPasswordKey$kCFProxyPortNumberKey$kCFProxyTypeAutoConfigurationJavaScript$kCFProxyTypeAutoConfigurationURL$kCFProxyTypeFTP$kCFProxyTypeHTTP$kCFProxyTypeHTTPS$kCFProxyTypeKey$kCFProxyTypeNone$kCFProxyTypeSOCKS$kCFProxyUsernameKey$kCFSOCKSNegotiationMethodKey$kCFSOCKSStatusCodeKey$kCFSOCKSVersionKey$kCFStreamErrorDomainFTP@i$kCFStreamErrorDomainHTTP@i$kCFStreamErrorDomainMach@i$kCFStreamErrorDomainNetDB@i$kCFStreamErrorDomainNetServices@i$kCFStreamErrorDomainSOCKS@i$kCFStreamErrorDomainSSL@i$kCFStreamErrorDomainSystemConfiguration@i$kCFStreamErrorDomainWinSock@q$kCFStreamNetworkServiceType$kCFStreamNetworkServiceTypeAVStreaming$kCFStreamNetworkServiceTypeBackground$kCFStreamNetworkServiceTypeCallSignaling$kCFStreamNetworkServiceTypeResponsiveAV$kCFStreamNetworkServiceTypeResponsiveData$kCFStreamNetworkServiceTypeVideo$kCFStreamNetworkServiceTypeVoIP$kCFStreamNetworkServiceTypeVoice$kCFStreamPropertyAllowConstrainedNetworkAccess$kCFStreamPropertyAllowExpensiveNetworkAccess$kCFStreamPropertyConnectionIsCellular$kCFStreamPropertyConnectionIsExpensive$kCFStreamPropertyFTPAttemptPersistentConnection$kCFStreamPropertyFTPFetchResourceInfo$kCFStreamPropertyFTPFileTransferOffset$kCFStreamPropertyFTPPassword$kCFStreamPropertyFTPProxy$kCFStreamPropertyFTPProxyHost$kCFStreamPropertyFTPProxyPassword$kCFStreamPropertyFTPProxyPort$kCFStreamPropertyFTPProxyUser$kCFStreamPropertyFTPResourceSize$kCFStreamPropertyFTPUsePassiveMode$kCFStreamPropertyFTPUserName$kCFStreamPropertyHTTPAttemptPersistentConnection$kCFStreamPropertyHTTPFinalRequest$kCFStreamPropertyHTTPFinalURL$kCFStreamPropertyHTTPProxy$kCFStreamPropertyHTTPProxyHost$kCFStreamPropertyHTTPProxyPort$kCFStreamPropertyHTTPRequestBytesWrittenCount$kCFStreamPropertyHTTPResponseHeader$kCFStreamPropertyHTTPSProxyHost$kCFStreamPropertyHTTPSProxyPort$kCFStreamPropertyHTTPShouldAutoredirect$kCFStreamPropertyNoCellular$kCFStreamPropertyProxyLocalBypass$kCFStreamPropertySOCKSPassword$kCFStreamPropertySOCKSProxy$kCFStreamPropertySOCKSProxyHost$kCFStreamPropertySOCKSProxyPort$kCFStreamPropertySOCKSUser$kCFStreamPropertySOCKSVersion$kCFStreamPropertySSLContext$kCFStreamPropertySSLPeerCertificates$kCFStreamPropertySSLPeerTrust$kCFStreamPropertySSLSettings$kCFStreamPropertyShouldCloseNativeSocket$kCFStreamPropertySocketExtendedBackgroundIdleMode$kCFStreamPropertySocketRemoteHost$kCFStreamPropertySocketRemoteNetService$kCFStreamPropertySocketSecurityLevel$kCFStreamSSLAllowsAnyRoot$kCFStreamSSLAllowsExpiredCertificates$kCFStreamSSLAllowsExpiredRoots$kCFStreamSSLCertificates$kCFStreamSSLIsServer$kCFStreamSSLLevel$kCFStreamSSLPeerName$kCFStreamSSLValidatesCertificateChain$kCFStreamSocketSOCKSVersion4$kCFStreamSocketSOCKSVersion5$kCFStreamSocketSecurityLevelNegotiatedSSL$kCFStreamSocketSecurityLevelNone$kCFStreamSocketSecurityLevelSSLv2$kCFStreamSocketSecurityLevelSSLv3$kCFStreamSocketSecurityLevelTLSv1$kCFURLErrorFailingURLErrorKey$kCFURLErrorFailingURLStringErrorKey$"""
enums = """$kCFErrorHTTPAuthenticationTypeUnsupported@300$kCFErrorHTTPBadCredentials@301$kCFErrorHTTPBadProxyCredentials@307$kCFErrorHTTPBadURL@305$kCFErrorHTTPConnectionLost@302$kCFErrorHTTPParseFailure@303$kCFErrorHTTPProxyConnectionFailure@306$kCFErrorHTTPRedirectionLoopDetected@304$kCFErrorHTTPSProxyConnectionFailure@310$kCFErrorPACFileAuth@309$kCFErrorPACFileError@308$kCFFTPErrorUnexpectedStatusCode@200$kCFHTTPCookieCannotParseCookieFile@-4000$kCFHostAddresses@0$kCFHostErrorHostNotFound@1$kCFHostErrorUnknown@2$kCFHostNames@1$kCFHostReachability@2$kCFNetDiagnosticConnectionDown@-66557$kCFNetDiagnosticConnectionIndeterminate@-66558$kCFNetDiagnosticConnectionUp@-66559$kCFNetDiagnosticErr@-66560$kCFNetDiagnosticNoErr@0$kCFNetServiceErrorBadArgument@-72004$kCFNetServiceErrorCancel@-72005$kCFNetServiceErrorCollision@-72001$kCFNetServiceErrorDNSServiceFailure@-73000$kCFNetServiceErrorInProgress@-72003$kCFNetServiceErrorInvalid@-72006$kCFNetServiceErrorNotFound@-72002$kCFNetServiceErrorTimeout@-72007$kCFNetServiceErrorUnknown@-72000$kCFNetServiceFlagIsDefault@4$kCFNetServiceFlagIsDomain@2$kCFNetServiceFlagIsRegistrationDomain@4$kCFNetServiceFlagMoreComing@1$kCFNetServiceFlagNoAutoRename@1$kCFNetServiceFlagRemove@8$kCFNetServiceMonitorTXT@1$kCFNetServicesErrorBadArgument@-72004$kCFNetServicesErrorCancel@-72005$kCFNetServicesErrorCollision@-72001$kCFNetServicesErrorInProgress@-72003$kCFNetServicesErrorInvalid@-72006$kCFNetServicesErrorMissingRequiredConfiguration@-72008$kCFNetServicesErrorNotFound@-72002$kCFNetServicesErrorTimeout@-72007$kCFNetServicesErrorUnknown@-72000$kCFSOCKS4ErrorIdConflict@112$kCFSOCKS4ErrorIdentdFailed@111$kCFSOCKS4ErrorRequestFailed@110$kCFSOCKS4ErrorUnknownStatusCode@113$kCFSOCKS5ErrorBadCredentials@122$kCFSOCKS5ErrorBadResponseAddr@121$kCFSOCKS5ErrorBadState@120$kCFSOCKS5ErrorNoAcceptableMethod@124$kCFSOCKS5ErrorUnsupportedNegotiationMethod@123$kCFSOCKSErrorUnknownClientVersion@100$kCFSOCKSErrorUnsupportedServerVersion@101$kCFStreamErrorHTTPAuthenticationBadPassword@-1002$kCFStreamErrorHTTPAuthenticationBadUserName@-1001$kCFStreamErrorHTTPAuthenticationTypeUnsupported@-1000$kCFStreamErrorHTTPBadURL@-3$kCFStreamErrorHTTPParseFailure@-1$kCFStreamErrorHTTPRedirectionLoop@-2$kCFStreamErrorHTTPSProxyFailureUnexpectedResponseToCONNECTMethod@311$kCFStreamErrorSOCKS4IdConflict@93$kCFStreamErrorSOCKS4IdentdFailed@92$kCFStreamErrorSOCKS4RequestFailed@91$kCFStreamErrorSOCKS4SubDomainResponse@2$kCFStreamErrorSOCKS5BadResponseAddr@1$kCFStreamErrorSOCKS5BadState@2$kCFStreamErrorSOCKS5SubDomainMethod@4$kCFStreamErrorSOCKS5SubDomainResponse@5$kCFStreamErrorSOCKS5SubDomainUserPass@3$kCFStreamErrorSOCKSSubDomainNone@0$kCFStreamErrorSOCKSSubDomainVersionCode@1$kCFStreamErrorSOCKSUnknownClientVersion@3$kCFStreamSocketSecurityNone@0$kCFStreamSocketSecuritySSLv2@1$kCFStreamSocketSecuritySSLv23@3$kCFStreamSocketSecuritySSLv3@2$kCFStreamSocketSecurityTLSv1@4$kCFURLErrorAppTransportSecurityRequiresSecureConnection@-1022$kCFURLErrorBackgroundSessionInUseByAnotherProcess@-996$kCFURLErrorBackgroundSessionWasDisconnected@-997$kCFURLErrorBadServerResponse@-1011$kCFURLErrorBadURL@-1000$kCFURLErrorCallIsActive@-1019$kCFURLErrorCancelled@-999$kCFURLErrorCannotCloseFile@-3002$kCFURLErrorCannotConnectToHost@-1004$kCFURLErrorCannotCreateFile@-3000$kCFURLErrorCannotDecodeContentData@-1016$kCFURLErrorCannotDecodeRawData@-1015$kCFURLErrorCannotFindHost@-1003$kCFURLErrorCannotLoadFromNetwork@-2000$kCFURLErrorCannotMoveFile@-3005$kCFURLErrorCannotOpenFile@-3001$kCFURLErrorCannotParseResponse@-1017$kCFURLErrorCannotRemoveFile@-3004$kCFURLErrorCannotWriteToFile@-3003$kCFURLErrorClientCertificateRejected@-1205$kCFURLErrorClientCertificateRequired@-1206$kCFURLErrorDNSLookupFailed@-1006$kCFURLErrorDataLengthExceedsMaximum@-1103$kCFURLErrorDataNotAllowed@-1020$kCFURLErrorDownloadDecodingFailedMidStream@-3006$kCFURLErrorDownloadDecodingFailedToComplete@-3007$kCFURLErrorFileDoesNotExist@-1100$kCFURLErrorFileIsDirectory@-1101$kCFURLErrorFileOutsideSafeArea@-1104$kCFURLErrorHTTPTooManyRedirects@-1007$kCFURLErrorInternationalRoamingOff@-1018$kCFURLErrorNetworkConnectionLost@-1005$kCFURLErrorNoPermissionsToReadFile@-1102$kCFURLErrorNotConnectedToInternet@-1009$kCFURLErrorRedirectToNonExistentLocation@-1010$kCFURLErrorRequestBodyStreamExhausted@-1021$kCFURLErrorResourceUnavailable@-1008$kCFURLErrorSecureConnectionFailed@-1200$kCFURLErrorServerCertificateHasBadDate@-1201$kCFURLErrorServerCertificateHasUnknownRoot@-1203$kCFURLErrorServerCertificateNotYetValid@-1204$kCFURLErrorServerCertificateUntrusted@-1202$kCFURLErrorTimedOut@-1001$kCFURLErrorUnknown@-998$kCFURLErrorUnsupportedURL@-1002$kCFURLErrorUserAuthenticationRequired@-1013$kCFURLErrorUserCancelledAuthentication@-1012$kCFURLErrorZeroByteResource@-1014$kSOCKS5NoAcceptableMethod@255$"""
misc.update(
    {
        "CFNetServicesError": NewType("CFNetServicesError", int),
        "CFNetServiceBrowserFlags": NewType("CFNetServiceBrowserFlags", int),
        "CFStreamErrorHTTP": NewType("CFStreamErrorHTTP", int),
        "CFNetServiceMonitorType": NewType("CFNetServiceMonitorType", int),
        "CFStreamErrorHTTPAuthentication": NewType(
            "CFStreamErrorHTTPAuthentication", int
        ),
        "CFNetServiceRegisterFlags": NewType("CFNetServiceRegisterFlags", int),
        "CFNetDiagnosticStatusValues": NewType("CFNetDiagnosticStatusValues", int),
        "CFHostInfoType": NewType("CFHostInfoType", int),
        "CFNetworkErrors": NewType("CFNetworkErrors", int),
    }
)
misc.update({})
functions = {
    "CFNetworkExecuteProxyAutoConfigurationURL": (
        b"^{__CFRunLoopSource=}^{__CFURL=}^{__CFURL=}^?^{_CFStreamClientContext=q^v^?^?^?}",
        "",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"^{__CFArray=}"},
                            2: {"type": b"^{__CFError=}"},
                        },
                    }
                }
            }
        },
    ),
    "CFHTTPAuthenticationRequiresOrderedRequests": (b"Z^{_CFHTTPAuthentication=}",),
    "CFHTTPAuthenticationCreateFromResponse": (
        b"^{_CFHTTPAuthentication=}^{__CFAllocator=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceBrowserGetTypeID": (b"Q",),
    "CFHostCreateWithName": (
        b"^{__CFHost=}^{__CFAllocator=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHostGetAddressing": (
        b"^{__CFArray=}^{__CFHost=}^Z",
        "",
        {"arguments": {1: {"type_modifier": "o"}}},
    ),
    "CFHTTPMessageSetBody": (b"v^{__CFHTTPMessage=}^{__CFData=}",),
    "CFHTTPAuthenticationGetTypeID": (b"Q",),
    "CFNetDiagnosticCreateWithStreams": (
        b"^{__CFNetDiagnostic=}^{__CFAllocator=}^{__CFReadStream=}^{__CFWriteStream=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFSocketStreamSOCKSGetError": (b"i^{_CFStreamError=qi}",),
    "CFHostCreateWithAddress": (
        b"^{__CFHost=}^{__CFAllocator=}^{__CFData=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceBrowserSearchForDomains": (
        b"Z^{__CFNetServiceBrowser=}Z^{_CFStreamError=qi}",
    ),
    "CFNetServiceUnscheduleFromRunLoop": (
        b"v^{__CFNetService=}^{__CFRunLoop=}^{__CFString=}",
    ),
    "CFNetServiceMonitorStop": (b"v^{__CFNetServiceMonitor=}^{_CFStreamError=qi}",),
    "CFNetworkCopySystemProxySettings": (
        b"^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHostGetReachability": (
        b"^{__CFData=}^{__CFHost=}^Z",
        "",
        {"arguments": {1: {"type_modifier": "o"}}},
    ),
    "CFHTTPMessageIsHeaderComplete": (b"Z^{__CFHTTPMessage=}",),
    "CFHTTPMessageGetTypeID": (b"Q",),
    "CFNetServiceMonitorGetTypeID": (b"Q",),
    "CFNetServiceGetPortNumber": (b"i^{__CFNetService=}",),
    "CFHTTPMessageCreateRequest": (
        b"^{__CFHTTPMessage=}^{__CFAllocator=}^{__CFString=}^{__CFURL=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceCreateCopy": (
        b"^{__CFNetService=}^{__CFAllocator=}^{__CFNetService=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceGetName": (b"^{__CFString=}^{__CFNetService=}",),
    "CFHTTPMessageCopyRequestMethod": (
        b"^{__CFString=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHTTPAuthenticationCopyRealm": (
        b"^{__CFString=}^{_CFHTTPAuthentication=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceMonitorInvalidate": (b"v^{__CFNetServiceMonitor=}",),
    "CFNetServiceGetProtocolSpecificInformation": (b"@^{__CFNetService=}",),
    "CFNetServiceCancel": (b"v^{__CFNetService=}",),
    "CFNetServiceMonitorCreate": (
        b"^{__CFNetServiceMonitor=}^{__CFAllocator=}^{__CFNetService=}^?^{CFNetServiceClientContext=q^v^?^?^?}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^{__CFNetServiceMonitor=}"},
                            1: {"type": b"^{__CFNetService=}"},
                            2: {"type": b"i"},
                            3: {"type": b"^{__CFData=}"},
                            4: {"type": b"^{_CFStreamError=qi}"},
                            5: {"type": b"^v"},
                        },
                    }
                }
            },
        },
    ),
    "CFWriteStreamCreateWithFTPURL": (
        b"^{__CFWriteStream=}^{__CFAllocator=}^{__CFURL=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceBrowserUnscheduleFromRunLoop": (
        b"v^{__CFNetServiceBrowser=}^{__CFRunLoop=}^{__CFString=}",
    ),
    "CFHTTPMessageCopyRequestURL": (
        b"^{__CFURL=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHTTPMessageCopyVersion": (
        b"^{__CFString=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceCreate": (
        b"^{__CFNetService=}^{__CFAllocator=}^{__CFString=}^{__CFString=}^{__CFString=}i",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceMonitorUnscheduleFromRunLoop": (
        b"v^{__CFNetServiceMonitor=}^{__CFRunLoop=}^{__CFString=}",
    ),
    "CFHostScheduleWithRunLoop": (b"v^{__CFHost=}^{__CFRunLoop=}^{__CFString=}",),
    "CFNetServiceMonitorStart": (b"Z^{__CFNetServiceMonitor=}i^{_CFStreamError=qi}",),
    "CFHostGetNames": (
        b"^{__CFArray=}^{__CFHost=}^Z",
        "",
        {"arguments": {1: {"type_modifier": "o"}}},
    ),
    "CFNetDiagnosticCopyNetworkStatusPassively": (
        b"q^{__CFNetDiagnostic=}^^{__CFString=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {1: {"already_cfretained": True, "type_modifier": "o"}},
        },
    ),
    "CFHTTPMessageCopyResponseStatusLine": (
        b"^{__CFString=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetDiagnosticDiagnoseProblemInteractively": (b"q^{__CFNetDiagnostic=}",),
    "CFHTTPMessageAddAuthentication": (
        b"Z^{__CFHTTPMessage=}^{__CFHTTPMessage=}^{__CFString=}^{__CFString=}^{__CFString=}Z",
    ),
    "CFNetworkExecuteProxyAutoConfigurationScript": (
        b"^{__CFRunLoopSource=}^{__CFString=}^{__CFURL=}^?^{_CFStreamClientContext=q^v^?^?^?}",
        "",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"^{__CFArray=}"},
                            2: {"type": b"^{__CFError=}"},
                        },
                    }
                }
            }
        },
    ),
    "CFNetDiagnosticCreateWithURL": (
        b"^{__CFNetDiagnostic=}^{__CFAllocator=}^{__CFURL=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceGetDomain": (b"^{__CFString=}^{__CFNetService=}",),
    "CFFTPCreateParsedResourceListing": (
        b"q^{__CFAllocator=}n^vq^^{__CFDictionary=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                1: {"c_array_length_in_arg": 2, "type_modifier": "n"},
                3: {"already_cfretained": True, "type_modifier": "o"},
            },
        },
    ),
    "CFNetServiceGetTypeID": (b"Q",),
    "CFHTTPMessageCreateEmpty": (
        b"^{__CFHTTPMessage=}^{__CFAllocator=}Z",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHTTPMessageAppendBytes": (
        b"Z^{__CFHTTPMessage=}^vq",
        "",
        {"arguments": {1: {"c_array_length_in_arg": 2, "type_modifier": "n"}}},
    ),
    "CFHostCancelInfoResolution": (b"v^{__CFHost=}i",),
    "CFNetServiceBrowserSearchForServices": (
        b"Z^{__CFNetServiceBrowser=}^{__CFString=}^{__CFString=}^{_CFStreamError=qi}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "CFHTTPAuthenticationIsValid": (
        b"Z^{_CFHTTPAuthentication=}^{_CFStreamError=qi}",
        "",
        {"arguments": {1: {"type_modifier": "o"}}},
    ),
    "CFHTTPAuthenticationAppliesToRequest": (
        b"Z^{_CFHTTPAuthentication=}^{__CFHTTPMessage=}",
    ),
    "CFNetServiceBrowserInvalidate": (b"v^{__CFNetServiceBrowser=}",),
    "CFNetServiceGetAddressing": (b"^{__CFArray=}^{__CFNetService=}",),
    "CFHTTPMessageSetHeaderFieldValue": (
        b"v^{__CFHTTPMessage=}^{__CFString=}^{__CFString=}",
    ),
    "CFNetServiceBrowserStopSearch": (
        b"v^{__CFNetServiceBrowser=}^{_CFStreamError=qi}",
    ),
    "CFHTTPMessageApplyCredentials": (
        b"Z^{__CFHTTPMessage=}^{_CFHTTPAuthentication=}^{__CFString=}^{__CFString=}^{_CFStreamError=qi}",
        "",
        {"arguments": {4: {"type_modifier": "o"}}},
    ),
    "CFHTTPReadStreamSetRedirectsAutomatically": (b"v^{__CFReadStream=}Z",),
    "CFNetServiceRegisterWithOptions": (
        b"Z^{__CFNetService=}Q^{_CFStreamError=qi}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "CFHTTPAuthenticationCopyMethod": (
        b"^{__CFString=}^{_CFHTTPAuthentication=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceBrowserCreate": (
        b"^{__CFNetServiceBrowser=}^{__CFAllocator=}^?^{CFNetServiceClientContext=q^v^?^?^?}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                1: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^{__CFNetServiceBrowser=}"},
                            1: {"type": b"Q"},
                            2: {"type": b"@"},
                            3: {"type": b"^{_CFStreamError=qi}"},
                            4: {"type": b"^v"},
                        },
                    }
                }
            },
        },
    ),
    "CFNetServiceGetTargetHost": (b"^{__CFString=}^{__CFNetService=}",),
    "CFNetworkCopyProxiesForAutoConfigurationScript": (
        b"^{__CFArray=}^{__CFString=}^{__CFURL=}^^{__CFError=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                2: {
                    "already_cfretained": True,
                    "type_modifier": "o",
                    "null_accepted": True,
                }
            },
        },
    ),
    "CFNetServiceCreateDictionaryWithTXTData": (
        b"^{__CFDictionary=}^{__CFAllocator=}^{__CFData=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceRegister": (b"Z^{__CFNetService=}^{_CFStreamError=qi}",),
    "CFHTTPMessageCopySerializedMessage": (
        b"^{__CFData=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHostCreateCopy": (
        b"^{__CFHost=}^{__CFAllocator=}^{__CFHost=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHTTPAuthenticationRequiresAccountDomain": (b"Z^{_CFHTTPAuthentication=}",),
    "CFNetServiceSetTXTData": (b"Z^{__CFNetService=}^{__CFData=}",),
    "CFNetworkCopyProxiesForURL": (
        b"^{__CFArray=}^{__CFURL=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFReadStreamCreateForStreamedHTTPRequest": (
        b"^{__CFReadStream=}^{__CFAllocator=}^{__CFHTTPMessage=}^{__CFReadStream=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFReadStreamCreateForHTTPRequest": (
        b"^{__CFReadStream=}^{__CFAllocator=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHTTPMessageCreateCopy": (
        b"^{__CFHTTPMessage=}^{__CFAllocator=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceGetType": (b"^{__CFString=}^{__CFNetService=}",),
    "CFNetServiceScheduleWithRunLoop": (
        b"v^{__CFNetService=}^{__CFRunLoop=}^{__CFString=}",
    ),
    "CFHTTPMessageCopyHeaderFieldValue": (
        b"^{__CFString=}^{__CFHTTPMessage=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceGetTXTData": (b"^{__CFData=}^{__CFNetService=}",),
    "CFHostStartInfoResolution": (
        b"Z^{__CFHost=}i^{_CFStreamError=qi}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "CFHTTPAuthenticationRequiresUserNameAndPassword": (b"Z^{_CFHTTPAuthentication=}",),
    "CFNetDiagnosticSetName": (b"v^{__CFNetDiagnostic=}^{__CFString=}",),
    "CFNetServiceCreateTXTDataWithDictionary": (
        b"^{__CFData=}^{__CFAllocator=}^{__CFDictionary=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFReadStreamCreateWithFTPURL": (
        b"^{__CFReadStream=}^{__CFAllocator=}^{__CFURL=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceSetClient": (
        b"Z^{__CFNetService=}^?^{CFNetServiceClientContext=q^v^?^?^?}",
        "",
        {
            "arguments": {
                1: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^{__CFNetService=}"},
                            1: {"type": b"^{_CFStreamError=qi}"},
                            2: {"type": b"^v"},
                        },
                    }
                }
            }
        },
    ),
    "CFNetServiceMonitorScheduleWithRunLoop": (
        b"v^{__CFNetServiceMonitor=}^{__CFRunLoop=}^{__CFString=}",
    ),
    "CFHostUnscheduleFromRunLoop": (b"v^{__CFHost=}^{__CFRunLoop=}^{__CFString=}",),
    "CFHTTPMessageApplyCredentialDictionary": (
        b"Z^{__CFHTTPMessage=}^{_CFHTTPAuthentication=}^{__CFDictionary=}^{_CFStreamError=qi}",
        "",
        {"arguments": {3: {"type_modifier": "o"}}},
    ),
    "CFHTTPMessageIsRequest": (b"Z^{__CFHTTPMessage=}",),
    "CFNetServiceResolve": (b"Z^{__CFNetService=}^{_CFStreamError=qi}",),
    "CFHTTPMessageCopyBody": (
        b"^{__CFData=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceBrowserScheduleWithRunLoop": (
        b"v^{__CFNetServiceBrowser=}^{__CFRunLoop=}^{__CFString=}",
    ),
    "CFHTTPMessageCreateResponse": (
        b"^{__CFHTTPMessage=}^{__CFAllocator=}q^{__CFString=}^{__CFString=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHostGetTypeID": (b"Q",),
    "CFHTTPMessageCopyAllHeaderFields": (
        b"^{__CFDictionary=}^{__CFHTTPMessage=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFHTTPMessageGetResponseStatusCode": (b"q^{__CFHTTPMessage=}",),
    "CFHTTPAuthenticationCopyDomains": (
        b"^{__CFArray=}^{_CFHTTPAuthentication=}",
        "",
        {"retval": {"already_cfretained": True}},
    ),
    "CFNetServiceSetProtocolSpecificInformation": (b"v^{__CFNetService=}@",),
    "CFSocketStreamSOCKSGetErrorSubdomain": (b"i^{_CFStreamError=qi}",),
    "CFStreamCreatePairWithSocketToCFHost": (
        b"v^{__CFAllocator=}^{__CFHost=}i^^{__CFReadStream=}^^{__CFWriteStream=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                3: {"already_cfretained": True, "type_modifier": "o"},
                4: {"already_cfretained": True, "type_modifier": "o"},
            },
        },
    ),
    "CFStreamCreatePairWithSocketToNetService": (
        b"v^{__CFAllocator=}^{__CFNetService=}^^{__CFReadStream=}^^{__CFWriteStream=}",
        "",
        {
            "retval": {"already_cfretained": True},
            "arguments": {
                2: {"already_cfretained": True, "type_modifier": "o"},
                3: {"already_cfretained": True, "type_modifier": "o"},
            },
        },
    ),
    "CFNetServiceResolveWithTimeout": (
        b"Z^{__CFNetService=}d^{_CFStreamError=qi}",
        "",
        {"arguments": {2: {"type_modifier": "o"}}},
    ),
    "CFHostSetClient": (
        b"Z^{__CFHost=}^?^{CFHostClientContext=q^v^?^?^?}",
        "",
        {
            "arguments": {
                1: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^{__CFHost=}"},
                            1: {"type": b"i"},
                            2: {"type": b"^{_CFStreamError=qi}"},
                            3: {"type": b"^v"},
                        },
                    }
                }
            }
        },
    ),
}
cftypes = [
    ("CFHTTPMessageRef", b"^{__CFHTTPMessage=}", "CFHTTPMessageGetTypeID", None),
    ("CFHostRef", b"^{__CFHost=}", "CFHostGetTypeID", None),
    ("CFNetDiagnosticRef", b"^{__CFNetDiagnostic=}", "CFNetDiagnosticGetTypeID", None),
    (
        "CFNetServiceBrowserRef",
        b"^{__CFNetServiceBrowser=}",
        "CFNetServiceBrowserGetTypeID",
        None,
    ),
    (
        "CFNetServiceMonitorRef",
        b"^{__CFNetServiceMonitor=}",
        "CFNetServiceMonitorGetTypeID",
        None,
    ),
    ("CFNetServiceRef", b"^{__CFNetService=}", "CFNetServiceGetTypeID", None),
    (
        "CFHTTPAuthenticationRef",
        b"^{_CFHTTPAuthentication}",
        "CFHTTPAuthenticationGetTypeID",
        None,
    ),
]
expressions = {}

# END OF FILE



import os

import CFNetwork
from PyObjCTools.TestSupport import TestCase, min_os_level, os_level_key, os_release


class TestCFHTTPStream(TestCase):
    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPFinalRequest, str)

    def testConstants(self):
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainHTTP, int)

        self.assertEqual(CFNetwork.kCFStreamErrorHTTPParseFailure, -1)
        self.assertEqual(CFNetwork.kCFStreamErrorHTTPRedirectionLoop, -2)
        self.assertEqual(CFNetwork.kCFStreamErrorHTTPBadURL, -3)

        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPResponseHeader, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPFinalURL, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPProxy, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPProxyHost, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPProxyPort, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPSProxyHost, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPSProxyPort, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyHTTPShouldAutoredirect, str)
        self.assertIsInstance(
            CFNetwork.kCFStreamPropertyHTTPAttemptPersistentConnection, str
        )
        self.assertIsInstance(
            CFNetwork.kCFStreamPropertyHTTPRequestBytesWrittenCount, str
        )

    def testFunctions(self):
        url = CFNetwork.CFURLCreateWithString(None, "http://www.python.org/", None)
        self.assertIsInstance(url, CFNetwork.CFURLRef)

        req = CFNetwork.CFHTTPMessageCreateRequest(
            None, "GET", url, CFNetwork.kCFHTTPVersion1_1
        )
        self.assertIsInstance(req, CFNetwork.CFHTTPMessageRef)

        self.assertResultIsCFRetained(CFNetwork.CFReadStreamCreateForHTTPRequest)
        v = CFNetwork.CFReadStreamCreateForHTTPRequest(None, req)
        self.assertIsInstance(v, CFNetwork.CFReadStreamRef)

        with open("/dev/null", "w") as fp:
            fd_2 = os.dup(2)
            os.dup2(fp.fileno(), 2)

        try:
            # Avoid deprecation messages from CFNetwork

            self.assertResultIsCFRetained(
                CFNetwork.CFReadStreamCreateForStreamedHTTPRequest
            )
            v = CFNetwork.CFReadStreamCreateForStreamedHTTPRequest(None, req, v)
            self.assertIsInstance(v, CFNetwork.CFReadStreamRef)

            if os_level_key(os_release()) < os_level_key("10.15"):
                self.assertArgIsBOOL(
                    CFNetwork.CFHTTPReadStreamSetRedirectsAutomatically, 1
                )
                CFNetwork.CFHTTPReadStreamSetRedirectsAutomatically(v, True)

            if os_level_key(os_release()) < os_level_key("10.9"):
                CFNetwork.CFHTTPReadStreamSetProxy(v, "localhost", 8080)

        finally:
            os.dup2(fd_2, 2)



import CFNetwork
from PyObjCTools.TestSupport import TestCase


class TestCFFTPStream(TestCase):
    def testConstants(self):
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainFTP, int)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPUserName, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPPassword, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPUsePassiveMode, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPResourceSize, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPFetchResourceInfo, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPFileTransferOffset, str)
        self.assertIsInstance(
            CFNetwork.kCFStreamPropertyFTPAttemptPersistentConnection, str
        )
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPProxy, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPProxyHost, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPProxyPort, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPProxyUser, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyFTPProxyPassword, str)
        self.assertIsInstance(CFNetwork.kCFFTPResourceMode, str)
        self.assertIsInstance(CFNetwork.kCFFTPResourceName, str)
        self.assertIsInstance(CFNetwork.kCFFTPResourceOwner, str)
        self.assertIsInstance(CFNetwork.kCFFTPResourceGroup, str)
        self.assertIsInstance(CFNetwork.kCFFTPResourceLink, str)
        self.assertIsInstance(CFNetwork.kCFFTPResourceSize, str)
        self.assertIsInstance(CFNetwork.kCFFTPResourceType, str)
        self.assertIsInstance(CFNetwork.kCFFTPResourceModDate, str)

    def testFunctions(self):
        self.assertResultIsCFRetained(CFNetwork.CFReadStreamCreateWithFTPURL)
        url = CFNetwork.CFURLCreateWithString(None, "ftp://ftp.python.org/", None)
        self.assertIsInstance(url, CFNetwork.CFURLRef)
        ftp = CFNetwork.CFReadStreamCreateWithFTPURL(None, url)
        self.assertIsInstance(ftp, CFNetwork.CFReadStreamRef)

        buf = b"-rw-r--r--  1 ronald  staff  1957 Mar 31 07:22 test_cfftpstream.py\r\n"
        self.assertArgHasType(CFNetwork.CFFTPCreateParsedResourceListing, 1, b"n^v")
        self.assertArgSizeInArg(CFNetwork.CFFTPCreateParsedResourceListing, 1, 2)
        self.assertArgIsOut(CFNetwork.CFFTPCreateParsedResourceListing, 3)
        cnt, out = CFNetwork.CFFTPCreateParsedResourceListing(None, buf, len(buf), None)
        self.assertIsInstance(cnt, int)
        self.assertIsInstance(out, CFNetwork.CFDictionaryRef)
        self.assertEqual(out[CFNetwork.kCFFTPResourceGroup], "staff")

        self.assertResultIsCFRetained(CFNetwork.CFWriteStreamCreateWithFTPURL)
        url = CFNetwork.CFURLCreateWithString(
            None, "ftp://www.rivm.nl/incoming/test.txt", None
        )
        self.assertIsInstance(url, CFNetwork.CFURLRef)
        ftp = CFNetwork.CFWriteStreamCreateWithFTPURL(None, url)
        self.assertIsInstance(ftp, CFNetwork.CFWriteStreamRef)



import CFNetwork
from Foundation import NSString
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCFFTPStream(TestCase):
    def testTypes(self):
        self.assertIsCFType(CFNetwork.CFHTTPAuthenticationRef)

    def testConstants(self):
        self.assertEqual(
            CFNetwork.kCFStreamErrorHTTPAuthenticationTypeUnsupported, -1000
        )
        self.assertEqual(CFNetwork.kCFStreamErrorHTTPAuthenticationBadUserName, -1001)
        self.assertEqual(CFNetwork.kCFStreamErrorHTTPAuthenticationBadPassword, -1002)

        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationUsername, str)
        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationPassword, str)
        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationAccountDomain, str)

    @min_os_level("10.5")
    # These functions should work on 10.4 as well, but caue a crash in
    # CFNetwork on that platform
    def testFunctions(self):
        self.assertIsInstance(CFNetwork.CFHTTPAuthenticationGetTypeID(), int)

        msg = CFNetwork.CFHTTPMessageCreateResponse(
            None, 401, "Authenticate", CFNetwork.kCFHTTPVersion1_0
        )
        self.assertIsInstance(msg, CFNetwork.CFHTTPMessageRef)
        CFNetwork.CFHTTPMessageSetHeaderFieldValue(
            msg,
            NSString.stringWithString_("WWW-Authenticate"),
            NSString.stringWithString_('Basic realm="WallyWorld"'),
        )

        self.assertResultIsCFRetained(CFNetwork.CFHTTPAuthenticationCreateFromResponse)
        ref = CFNetwork.CFHTTPAuthenticationCreateFromResponse(None, msg)
        self.assertIsInstance(ref, CFNetwork.CFHTTPAuthenticationRef)

        self.assertResultIsBOOL(CFNetwork.CFHTTPAuthenticationIsValid)
        self.assertArgIsOut(CFNetwork.CFHTTPAuthenticationIsValid, 1)
        v, err = CFNetwork.CFHTTPAuthenticationIsValid(ref, None)
        self.assertIsInstance(v, bool)
        if v:
            self.assertTrue(err is None)
        else:
            self.assertIsInstance(err, CFNetwork.CFStreamError)

        self.assertResultIsBOOL(CFNetwork.CFHTTPAuthenticationAppliesToRequest)
        v = CFNetwork.CFHTTPAuthenticationAppliesToRequest(ref, msg)
        self.assertIsInstance(v, bool)

        self.assertResultIsBOOL(CFNetwork.CFHTTPAuthenticationRequiresOrderedRequests)
        v = CFNetwork.CFHTTPAuthenticationRequiresOrderedRequests(ref)
        self.assertIsInstance(v, bool)

        url = CFNetwork.CFURLCreateWithString(None, "http://www.python.org/", None)
        req = CFNetwork.CFHTTPMessageCreateRequest(None, "GET", url, "1.0")

        self.assertResultIsBOOL(CFNetwork.CFHTTPMessageApplyCredentials)
        self.assertArgIsOut(CFNetwork.CFHTTPMessageApplyCredentials, 4)
        v, err = CFNetwork.CFHTTPMessageApplyCredentials(
            req, ref, "ronald", "secret", None
        )
        if v is True:
            self.assertEqual(err, None)

        else:
            self.assertIsInstance(err, CFNetwork.CFStreamError)

        self.assertResultIsBOOL(CFNetwork.CFHTTPMessageApplyCredentialDictionary)
        self.assertArgIsOut(CFNetwork.CFHTTPMessageApplyCredentialDictionary, 3)
        v, err = CFNetwork.CFHTTPMessageApplyCredentialDictionary(
            req,
            ref,
            {
                CFNetwork.kCFHTTPAuthenticationUsername: "ronald",
                CFNetwork.kCFHTTPAuthenticationPassword: "secret",
            },
            None,
        )
        if v is True:
            self.assertEqual(err, None)
        else:
            self.assertIsInstance(err, CFNetwork.CFStreamError)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPAuthenticationCopyRealm)
        self.assertResultHasType(
            CFNetwork.CFHTTPAuthenticationCopyRealm, b"^{__CFString=}"
        )
        v = CFNetwork.CFHTTPAuthenticationCopyRealm(ref)
        self.assertTrue(v is None or isinstance(v, str))

        self.assertResultIsCFRetained(CFNetwork.CFHTTPAuthenticationCopyDomains)
        self.assertResultHasType(
            CFNetwork.CFHTTPAuthenticationCopyDomains, b"^{__CFArray=}"
        )
        v = CFNetwork.CFHTTPAuthenticationCopyDomains(ref)
        self.assertTrue(v is None or isinstance(v, CFNetwork.CFArrayRef))

        self.assertResultIsCFRetained(CFNetwork.CFHTTPAuthenticationCopyMethod)
        self.assertResultHasType(
            CFNetwork.CFHTTPAuthenticationCopyMethod, b"^{__CFString=}"
        )
        v = CFNetwork.CFHTTPAuthenticationCopyMethod(ref)
        self.assertTrue(v is None or isinstance(v, str))

        self.assertResultIsBOOL(
            CFNetwork.CFHTTPAuthenticationRequiresUserNameAndPassword
        )
        v = CFNetwork.CFHTTPAuthenticationRequiresUserNameAndPassword(ref)
        self.assertIsInstance(v, bool)

        self.assertResultIsBOOL(CFNetwork.CFHTTPAuthenticationRequiresAccountDomain)
        v = CFNetwork.CFHTTPAuthenticationRequiresAccountDomain(ref)
        self.assertIsInstance(v, bool)



import Foundation
import CFNetwork
from PyObjCTools.TestSupport import TestCase, expectedFailure


class TestCFNetServices(TestCase):
    def testTypes(self):
        self.assertIsCFType(CFNetwork.CFNetServiceRef)
        self.assertIsCFType(CFNetwork.CFNetServiceMonitorRef)
        self.assertIsCFType(CFNetwork.CFNetServiceBrowserRef)

    def testConstants(self):
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainMach, int)
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainNetServices, int)

        self.assertEqual(CFNetwork.kCFNetServicesErrorUnknown, -72000)
        self.assertEqual(CFNetwork.kCFNetServicesErrorCollision, -72001)
        self.assertEqual(CFNetwork.kCFNetServicesErrorNotFound, -72002)
        self.assertEqual(CFNetwork.kCFNetServicesErrorInProgress, -72003)
        self.assertEqual(CFNetwork.kCFNetServicesErrorBadArgument, -72004)
        self.assertEqual(CFNetwork.kCFNetServicesErrorCancel, -72005)
        self.assertEqual(CFNetwork.kCFNetServicesErrorInvalid, -72006)
        self.assertEqual(CFNetwork.kCFNetServicesErrorTimeout, -72007)
        self.assertEqual(
            CFNetwork.kCFNetServicesErrorMissingRequiredConfiguration, -72008
        )
        self.assertEqual(CFNetwork.kCFNetServiceMonitorTXT, 1)
        self.assertEqual(CFNetwork.kCFNetServiceFlagNoAutoRename, 1)
        self.assertEqual(CFNetwork.kCFNetServiceFlagMoreComing, 1)
        self.assertEqual(CFNetwork.kCFNetServiceFlagIsDomain, 2)
        self.assertEqual(CFNetwork.kCFNetServiceFlagIsDefault, 4)
        self.assertEqual(CFNetwork.kCFNetServiceFlagIsRegistrationDomain, 4)
        self.assertEqual(CFNetwork.kCFNetServiceFlagRemove, 8)

    def testFunctions(self):
        self.assertIsInstance(CFNetwork.CFNetServiceGetTypeID(), int)
        self.assertIsInstance(CFNetwork.CFNetServiceMonitorGetTypeID(), int)
        self.assertIsInstance(CFNetwork.CFNetServiceBrowserGetTypeID(), int)

        self.assertResultIsCFRetained(CFNetwork.CFNetServiceCreate)
        serv = CFNetwork.CFNetServiceCreate(
            None, "pyobjc.local", "ssh", "pyobjc.test.local", 9999
        )
        self.assertIsInstance(serv, CFNetwork.CFNetServiceRef)

        self.assertResultIsCFRetained(CFNetwork.CFNetServiceCreateCopy)
        v = CFNetwork.CFNetServiceCreateCopy(None, serv)
        self.assertIsInstance(v, CFNetwork.CFNetServiceRef)

        dom = CFNetwork.CFNetServiceGetDomain(serv)
        self.assertIsInstance(dom, str)

        dom = CFNetwork.CFNetServiceGetType(serv)
        self.assertIsInstance(dom, str)

        dom = CFNetwork.CFNetServiceGetName(serv)
        self.assertIsInstance(dom, str)

        self.assertResultIsBOOL(CFNetwork.CFNetServiceRegisterWithOptions)
        self.assertArgIsOut(CFNetwork.CFNetServiceRegisterWithOptions, 2)

        ok, err = CFNetwork.CFNetServiceRegisterWithOptions(
            serv, CFNetwork.kCFNetServiceFlagNoAutoRename, None
        )
        self.assertIsInstance(ok, bool)
        if ok:
            self.assertEqual(err, None)
        else:
            self.assertIsInstance(err, CFNetwork.CFStreamError)

        self.assertResultIsBOOL(CFNetwork.CFNetServiceResolveWithTimeout)
        self.assertArgIsOut(CFNetwork.CFNetServiceResolveWithTimeout, 2)
        ok, err = CFNetwork.CFNetServiceResolveWithTimeout(serv, 1.0, None)
        self.assertIsInstance(ok, bool)
        if ok:
            self.assertEqual(err, None)
        else:
            self.assertIsInstance(err, CFNetwork.CFStreamError)

        host = CFNetwork.CFNetServiceGetTargetHost(serv)
        self.assertIsInstance(host, (str, type(None)))

        port = CFNetwork.CFNetServiceGetPortNumber(serv)
        self.assertIsInstance(port, int)

        v = CFNetwork.CFNetServiceGetAddressing(serv)
        self.assertIsInstance(v, (Foundation.NSArray, type(None)))

        v = CFNetwork.CFNetServiceGetTXTData(serv)
        self.assertIsInstance(v, (Foundation.NSData, type(None)))

        v = CFNetwork.CFNetServiceCreateTXTDataWithDictionary(
            None, {"key": "value", "key2": "value2"}
        )
        self.assertIsInstance(v, Foundation.NSData)

        v = CFNetwork.CFNetServiceCreateDictionaryWithTXTData(None, v)
        self.assertIsInstance(v, Foundation.NSDictionary)

        self.assertResultIsBOOL(CFNetwork.CFNetServiceSetTXTData)
        ok = CFNetwork.CFNetServiceSetTXTData(serv, b"hello")
        self.assertIsInstance(ok, bool)

        rl = CFNetwork.CFRunLoopGetCurrent()
        CFNetwork.CFNetServiceScheduleWithRunLoop(
            serv, rl, CFNetwork.kCFRunLoopDefaultMode
        )
        CFNetwork.CFNetServiceUnscheduleFromRunLoop(
            serv, rl, CFNetwork.kCFRunLoopDefaultMode
        )

        CFNetwork.CFNetServiceCancel(serv)

    @expectedFailure
    def testMissingTests(self):
        self.fail("CFNetServiceSetClient")
        self.fail("CFNetServiceMonitorCreate")
        self.fail("CFNetServiceMonitorInvalidate")
        self.fail("CFNetServiceMonitorStart")
        self.fail("CFNetServiceMonitorStop")
        self.fail("CFNetServiceMonitorScheduleWithRunLoop")
        self.fail("CFNetServiceMonitorUnscheduleFromRunLoop")
        self.fail("CFNetServiceBrowserCreate")
        self.fail("CFNetServiceBrowserInvalidate")
        self.fail("CFNetServiceBrowserSearchForDomains")
        self.fail("CFNetServiceBrowserSearchForServices")
        self.fail("CFNetServiceBrowserStopSearch")
        self.fail("CFNetServiceBrowserScheduleWithRunLoop")
        self.fail("CFNetServiceBrowserUnscheduleFromRunLoop")
        self.fail("CFNetServiceRegister")
        self.fail("CFNetServiceResolve")
        self.fail("CFNetServiceGetProtocolSpecificInformation")
        self.fail("CFNetServiceSetProtocolSpecificInformation")



import os

import CFNetwork
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure

SCRIPT = """
function FindProxyForURL(url, host) {
   if (shExpMatch(host, "*.apple.com")) {
      return "PROXY proxy.apple.com:8080";
   }
   return "DIRECT";
}
"""


class TestCFProxySupport(TestCase):
    @min_os_level("10.5")
    def testFunctions(self):
        self.assertResultIsCFRetained(CFNetwork.CFNetworkCopyProxiesForURL)
        url = CFNetwork.CFURLCreateWithString(None, "http://www.apple.com/", None)
        v = CFNetwork.CFNetworkCopyProxiesForURL(url, None)
        self.assertIsInstance(v, CFNetwork.CFArrayRef)

        self.assertResultIsCFRetained(
            CFNetwork.CFNetworkCopyProxiesForAutoConfigurationScript
        )
        self.assertArgIsOut(CFNetwork.CFNetworkCopyProxiesForAutoConfigurationScript, 2)
        v, err = CFNetwork.CFNetworkCopyProxiesForAutoConfigurationScript(
            SCRIPT, url, None
        )
        self.assertTrue(err is None)
        self.assertIsInstance(v, CFNetwork.CFArrayRef)
        x = v[0]
        self.assertEqual(x[CFNetwork.kCFProxyTypeKey], CFNetwork.kCFProxyTypeHTTP)
        self.assertEqual(x[CFNetwork.kCFProxyHostNameKey], "proxy.apple.com")
        self.assertEqual(x[CFNetwork.kCFProxyPortNumberKey], 8080)

        self.assertResultIsCFRetained(CFNetwork.CFNetworkCopySystemProxySettings)
        v = CFNetwork.CFNetworkCopySystemProxySettings()
        self.assertIsInstance(v, CFNetwork.CFDictionaryRef)

    @min_os_level("10.5")
    def testManual(self):
        lst = []
        ctx = object()

        def callback(ctx, proxies, error):
            lst.append([ctx, proxies, error])

        url = CFNetwork.CFURLCreateWithString(None, "http://www.apple.com/", None)

        rls = CFNetwork.CFNetworkExecuteProxyAutoConfigurationScript(
            SCRIPT, url, callback, ctx
        )
        self.assertIsInstance(rls, CFNetwork.CFRunLoopSourceRef)

        rl = CFNetwork.CFRunLoopGetCurrent()
        CFNetwork.CFRunLoopAddSource(rl, rls, CFNetwork.kCFRunLoopCommonModes)

        CFNetwork.CFRunLoopRunInMode(CFNetwork.kCFRunLoopDefaultMode, 1.0, False)

        CFNetwork.CFRunLoopRemoveSource(rl, rls, CFNetwork.kCFRunLoopCommonModes)

        self.assertNotEqual(len(lst), 0)
        self.assertTrue(lst[0][0] is ctx)
        self.assertIsInstance(lst[0][1], CFNetwork.CFArrayRef)
        self.assertEqual(lst[0][2], None)

        lst[:] = []
        path = os.path.join(os.path.dirname(__file__), "proxy.pac")
        cwd = os.getcwd()
        if path.startswith(cwd):
            path = path[len(cwd) + 1 :]
        scriptURL = CFNetwork.CFURLCreateWithFileSystemPath(
            None, path, CFNetwork.kCFURLPOSIXPathStyle, False
        )

        rls = CFNetwork.CFNetworkExecuteProxyAutoConfigurationURL(
            scriptURL, url, callback, ctx
        )
        self.assertIsInstance(rls, CFNetwork.CFRunLoopSourceRef)

        CFNetwork.CFRunLoopAddSource(rl, rls, CFNetwork.kCFRunLoopCommonModes)

        CFNetwork.CFRunLoopRunInMode(CFNetwork.kCFRunLoopDefaultMode, 1.0, True)

        CFNetwork.CFRunLoopRemoveSource(rl, rls, CFNetwork.kCFRunLoopCommonModes)

        # print lst

        self.assertNotEqual(len(lst), 0)
        self.assertTrue(lst[0][0] is ctx)
        if lst[0][2] is None:
            self.assertIsInstance(lst[0][1], CFNetwork.CFArrayRef)
            self.assertEqual(lst[0][2], None)

        else:
            self.assertEqual(lst[0][1], None)
            self.assertIsInstance(lst[0][2], CFNetwork.CFErrorRef)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(CFNetwork.kCFProxyAutoConfigurationJavaScriptKey, str)
        self.assertIsInstance(CFNetwork.kCFProxyTypeAutoConfigurationJavaScript, str)

    @expectedFailure
    @min_os_level("10.7")
    def testConstants10_7_failure(self):
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesProxyAutoConfigJavaScript, str)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(CFNetwork.kCFProxyTypeKey, str)
        self.assertIsInstance(CFNetwork.kCFProxyHostNameKey, str)
        self.assertIsInstance(CFNetwork.kCFProxyPortNumberKey, str)
        self.assertIsInstance(CFNetwork.kCFProxyAutoConfigurationURLKey, str)
        self.assertIsInstance(CFNetwork.kCFProxyUsernameKey, str)
        self.assertIsInstance(CFNetwork.kCFProxyPasswordKey, str)
        self.assertIsInstance(CFNetwork.kCFProxyTypeNone, str)
        self.assertIsInstance(CFNetwork.kCFProxyTypeHTTP, str)
        self.assertIsInstance(CFNetwork.kCFProxyTypeHTTPS, str)
        self.assertIsInstance(CFNetwork.kCFProxyTypeSOCKS, str)
        self.assertIsInstance(CFNetwork.kCFProxyTypeFTP, str)
        self.assertIsInstance(CFNetwork.kCFProxyTypeAutoConfigurationURL, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesHTTPEnable, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesHTTPPort, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesHTTPProxy, str)

    @expectedFailure
    @min_os_level("10.5")
    def testConstants_fail(self):
        self.assertHasAttr(CFNetwork, "kCFProxyAutoConfigurationHTTPResponseKey")
        self.assertIsInstance(CFNetwork.kCFProxyAutoConfigurationHTTPResponseKey, str)

    @min_os_level("10.6")
    def testFunctions10_6(self):
        r = CFNetwork.CFNetworkCopySystemProxySettings()
        self.assertIsInstance(r, CFNetwork.CFDictionaryRef)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesExceptionsList, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesExcludeSimpleHostnames, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesFTPEnable, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesFTPPassive, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesFTPPort, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesFTPProxy, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesGopherEnable, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesGopherPort, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesGopherProxy, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesHTTPSEnable, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesHTTPSPort, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesHTTPSProxy, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesRTSPEnable, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesRTSPPort, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesRTSPProxy, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesSOCKSEnable, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesSOCKSPort, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesSOCKSProxy, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesProxyAutoConfigEnable, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesProxyAutoConfigURLString, str)
        self.assertIsInstance(CFNetwork.kCFNetworkProxiesProxyAutoDiscoveryEnable, str)



import CFNetwork
from PyObjCTools.TestSupport import TestCase, min_os_level, os_release, os_level_key


class TestCFSocketStream(TestCase):
    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertIsInstance(
            CFNetwork.kCFStreamPropertyAllowExpensiveNetworkAccess, str
        )
        self.assertIsInstance(CFNetwork.kCFStreamPropertyConnectionIsExpensive, str)
        self.assertIsInstance(
            CFNetwork.kCFStreamPropertyAllowConstrainedNetworkAccess, str
        )

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceTypeCallSignaling, str)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(CFNetwork.kCFStreamPropertySSLContext, str)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(CFNetwork.kCFStreamPropertyNoCellular, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyConnectionIsCellular, str)
        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceTypeResponsiveData, str)

        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceTypeAVStreaming, str)
        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceTypeResponsiveAV, str)

        self.assertIsInstance(CFNetwork.kCFStreamPropertyNoCellular, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyConnectionIsCellular, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceType, str)
        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceTypeVoIP, str)
        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceTypeVideo, str)
        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceTypeBackground, str)
        self.assertIsInstance(CFNetwork.kCFStreamNetworkServiceTypeVoice, str)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(CFNetwork.kCFStreamPropertySSLPeerTrust, str)
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainWinSock, int)

    def testConstants(self):
        self.assertIsInstance(CFNetwork.kCFStreamPropertySSLPeerCertificates, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySSLSettings, str)
        self.assertIsInstance(CFNetwork.kCFStreamSSLLevel, str)
        self.assertIsInstance(CFNetwork.kCFStreamSSLAllowsExpiredCertificates, str)
        self.assertIsInstance(CFNetwork.kCFStreamSSLAllowsExpiredRoots, str)
        self.assertIsInstance(CFNetwork.kCFStreamSSLAllowsAnyRoot, str)
        self.assertIsInstance(CFNetwork.kCFStreamSSLValidatesCertificateChain, str)
        self.assertIsInstance(CFNetwork.kCFStreamSSLPeerName, str)
        self.assertIsInstance(CFNetwork.kCFStreamSSLCertificates, str)
        self.assertIsInstance(CFNetwork.kCFStreamSSLIsServer, str)

        self.assertEqual(CFNetwork.kCFStreamErrorSOCKSSubDomainNone, 0)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKSSubDomainVersionCode, 1)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS4SubDomainResponse, 2)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS5SubDomainUserPass, 3)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS5SubDomainMethod, 4)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS5SubDomainResponse, 5)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS5BadResponseAddr, 1)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS5BadState, 2)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKSUnknownClientVersion, 3)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS4RequestFailed, 91)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS4IdentdFailed, 92)
        self.assertEqual(CFNetwork.kCFStreamErrorSOCKS4IdConflict, 93)
        self.assertEqual(CFNetwork.kSOCKS5NoAcceptableMethod, 0xFF)

        # Moved to CoreFoundation in 10.14, still testing here for backward
        # compat reasons.
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainSOCKS, int)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySOCKSProxy, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySOCKSProxyHost, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySOCKSProxyPort, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySOCKSVersion, str)
        self.assertIsInstance(CFNetwork.kCFStreamSocketSOCKSVersion4, str)
        self.assertIsInstance(CFNetwork.kCFStreamSocketSOCKSVersion5, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySOCKSUser, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySOCKSPassword, str)
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainSSL, int)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySocketSecurityLevel, str)
        self.assertIsInstance(CFNetwork.kCFStreamSocketSecurityLevelNone, str)
        self.assertIsInstance(CFNetwork.kCFStreamSocketSecurityLevelSSLv2, str)
        self.assertIsInstance(CFNetwork.kCFStreamSocketSecurityLevelSSLv3, str)
        self.assertIsInstance(CFNetwork.kCFStreamSocketSecurityLevelTLSv1, str)
        self.assertIsInstance(CFNetwork.kCFStreamSocketSecurityLevelNegotiatedSSL, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertyShouldCloseNativeSocket, str)
        #

        self.assertIsInstance(CFNetwork.kCFStreamPropertySocketRemoteHost, str)
        self.assertIsInstance(CFNetwork.kCFStreamPropertySocketRemoteNetService, str)
        self.assertEqual(CFNetwork.kCFStreamSocketSecurityNone, 0)
        self.assertEqual(CFNetwork.kCFStreamSocketSecuritySSLv2, 1)
        self.assertEqual(CFNetwork.kCFStreamSocketSecuritySSLv3, 2)
        self.assertEqual(CFNetwork.kCFStreamSocketSecuritySSLv23, 3)
        self.assertEqual(CFNetwork.kCFStreamSocketSecurityTLSv1, 4)

        self.assertIsInstance(CFNetwork.kCFStreamPropertyProxyLocalBypass, str)

    def testFunctions(self):
        err = CFNetwork.CFStreamError()
        err.error = (1 << 16) + 2

        v = CFNetwork.CFSocketStreamSOCKSGetErrorSubdomain(err)
        self.assertEqual(v, 1)

        v = CFNetwork.CFSocketStreamSOCKSGetError(err)
        self.assertEqual(v, 2)

        self.assertArgIsCFRetained(CFNetwork.CFStreamCreatePairWithSocketToCFHost, 3)
        self.assertArgIsCFRetained(CFNetwork.CFStreamCreatePairWithSocketToCFHost, 4)
        self.assertArgIsOut(CFNetwork.CFStreamCreatePairWithSocketToCFHost, 3)
        self.assertArgIsOut(CFNetwork.CFStreamCreatePairWithSocketToCFHost, 4)

        host = CFNetwork.CFHostCreateWithName(None, "connect.apple.com")
        rd, wr = CFNetwork.CFStreamCreatePairWithSocketToCFHost(
            None, host, 443, None, None
        )
        self.assertIsInstance(rd, CFNetwork.CFReadStreamRef)
        self.assertIsInstance(wr, CFNetwork.CFWriteStreamRef)

        if os_level_key(os_release()) < os_level_key("10.9"):
            self.assertResultIsBOOL(CFNetwork.CFSocketStreamPairSetSecurityProtocol)
            v = CFNetwork.CFSocketStreamPairSetSecurityProtocol(
                rd, wr, CFNetwork.kCFStreamSocketSecuritySSLv23
            )
            self.assertIsInstance(v, bool)

        self.assertArgIsCFRetained(
            CFNetwork.CFStreamCreatePairWithSocketToNetService, 2
        )
        self.assertArgIsCFRetained(
            CFNetwork.CFStreamCreatePairWithSocketToNetService, 3
        )
        self.assertArgIsOut(CFNetwork.CFStreamCreatePairWithSocketToNetService, 2)
        self.assertArgIsOut(CFNetwork.CFStreamCreatePairWithSocketToNetService, 3)
        service = CFNetwork.CFNetServiceCreate(
            None, "pyobjc.local", "ssh", "pyobjc.test.local", 9999
        )
        rd, wr = CFNetwork.CFStreamCreatePairWithSocketToNetService(
            None, service, None, None
        )
        self.assertIsInstance(rd, CFNetwork.CFReadStreamRef)
        self.assertIsInstance(wr, CFNetwork.CFWriteStreamRef)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertIsInstance(
            CFNetwork.kCFStreamPropertySocketExtendedBackgroundIdleMode, str
        )



import socket

import CFNetwork
import objc
from PyObjCTools.TestSupport import TestCase


class TestCFHost(TestCase):
    def testTypes(self):
        self.assertIsCFType(CFNetwork.CFHostRef)

    def testConstants(self):
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainNetDB, int)
        self.assertIsInstance(CFNetwork.kCFStreamErrorDomainSystemConfiguration, int)
        self.assertEqual(CFNetwork.kCFHostAddresses, 0)
        self.assertEqual(CFNetwork.kCFHostNames, 1)
        self.assertEqual(CFNetwork.kCFHostReachability, 2)

    def testFunctions(self):
        self.assertIsInstance(CFNetwork.CFHostGetTypeID(), int)

        self.assertResultIsCFRetained(CFNetwork.CFHostCreateWithName)
        v = CFNetwork.CFHostCreateWithName(None, "www.python.org")
        self.assertIsInstance(v, CFNetwork.CFHostRef)

        try:
            # value = socket.gethostbyname("www.python.org")
            socket.gethostbyname("www.python.org")
            expected_resolution = True
        except OSError:
            expected_resolution = False

        addr = b" " * 24
        self.assertResultIsCFRetained(CFNetwork.CFHostCreateWithAddress)
        t = CFNetwork.CFHostCreateWithAddress(None, addr)
        self.assertIsInstance(t, CFNetwork.CFHostRef)

        self.assertResultIsBOOL(CFNetwork.CFHostStartInfoResolution)
        self.assertArgIsOut(CFNetwork.CFHostStartInfoResolution, 2)
        ok, error = CFNetwork.CFHostStartInfoResolution(
            v, CFNetwork.kCFHostAddresses, None
        )
        self.assertIs(ok, expected_resolution)
        self.assertIsInstance(error, CFNetwork.CFStreamError)

        self.assertResultIsCFRetained(CFNetwork.CFHostCreateCopy)
        # w = CFNetwork.CFHostCreateCopy(None, v)
        # self.assertIsInstance(w, type(v))

        self.assertArgHasType(
            CFNetwork.CFHostGetReachability, 1, b"o^" + objc._C_NSBOOL
        )
        lst, ok = CFNetwork.CFHostGetReachability(v, None)
        self.assertIsInstance(lst, (CFNetwork.CFDataRef, type(None)))
        self.assertIsInstance(ok, bool)

        CFNetwork.CFHostCancelInfoResolution(v, CFNetwork.kCFHostAddresses)

        rl = CFNetwork.CFRunLoopGetCurrent()
        CFNetwork.CFHostScheduleWithRunLoop(v, rl, CFNetwork.kCFRunLoopDefaultMode)

        CFNetwork.CFHostUnscheduleFromRunLoop(v, rl, CFNetwork.kCFRunLoopDefaultMode)

        self.assertArgHasType(CFNetwork.CFHostGetNames, 1, b"o^" + objc._C_NSBOOL)
        lst, ok = CFNetwork.CFHostGetNames(v, None)
        # print("XXX", lst)
        self.assertIsInstance(lst, CFNetwork.CFArrayRef)
        self.assertIsInstance(ok, bool)
        # self.assertIn(value, lst)

    def testCallbacks(self):
        lst = []
        ctx = object()

        def callback(host, typeinfo, error, ctx):
            lst.append([host, typeinfo, error, ctx])

        host = CFNetwork.CFHostCreateWithName(None, "localhost")
        CFNetwork.CFHostSetClient(host, callback, ctx)

        rl = CFNetwork.CFRunLoopGetCurrent()
        CFNetwork.CFHostScheduleWithRunLoop(host, rl, CFNetwork.kCFRunLoopDefaultMode)

        ok, err = CFNetwork.CFHostStartInfoResolution(
            host, CFNetwork.kCFHostAddresses, None
        )
        self.assertTrue(ok)
        self.assertIsInstance(ok, bool)
        self.assertIsInstance(err, CFNetwork.CFStreamError)

        CFNetwork.CFRunLoopRunInMode(CFNetwork.kCFRunLoopDefaultMode, 4.0, False)

        CFNetwork.CFHostUnscheduleFromRunLoop(host, rl, CFNetwork.kCFRunLoopDefaultMode)
        self.assertEqual(len(lst), 1)
        self.assertIsInstance(lst[0][0], CFNetwork.CFHostRef)
        self.assertIsInstance(lst[0][1], int)
        self.assertIsInstance(lst[0][2], CFNetwork.CFStreamError)
        self.assertIs(lst[0][3], ctx)

        self.assertResultIsNotCFRetained(CFNetwork.CFHostGetAddressing)
        self.assertArgHasType(CFNetwork.CFHostGetAddressing, 1, b"o^Z")
        lst, ok = CFNetwork.CFHostGetAddressing(host, None)
        self.assertIsInstance(lst, CFNetwork.CFArrayRef)
        self.assertIsInstance(lst[0], CFNetwork.CFDataRef)
        self.assertIsInstance(ok, bool)



import CFNetwork
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestCFHTTPMessage(TestCase):
    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationSchemeNegotiate2, str)
        self.assertIsInstance(
            CFNetwork.kCFHTTPAuthenticationSchemeXMobileMeAuthToken, str
        )

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationSchemeNTLM, str)
        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationSchemeNegotiate, str)

    @expectedFailure
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(CFNetwork.kCFHTTPVersion2_0, str)
        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationSchemeKerberos, str)

    @min_os_level("11.0")
    def testConstants11_0(self):
        self.assertIsInstance(CFNetwork.kCFHTTPVersion3_0, str)

    def testConstants(self):
        self.assertIsInstance(CFNetwork.kCFHTTPVersion1_0, str)
        self.assertIsInstance(CFNetwork.kCFHTTPVersion1_1, str)
        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationSchemeBasic, str)
        self.assertIsInstance(CFNetwork.kCFHTTPAuthenticationSchemeDigest, str)

    def testTypes(self):
        self.assertIsCFType(CFNetwork.CFHTTPMessageRef)

    def testFunctions(self):
        self.assertIsInstance(CFNetwork.CFHTTPMessageGetTypeID(), int)

        url = CFNetwork.CFURLCreateWithString(None, "http://www.python.org/", None)
        self.assertIsInstance(url, CFNetwork.CFURLRef)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCreateRequest)
        req = CFNetwork.CFHTTPMessageCreateRequest(
            None, "GET", url, CFNetwork.kCFHTTPVersion1_1
        )
        self.assertIsInstance(req, CFNetwork.CFHTTPMessageRef)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCreateResponse)
        resp = CFNetwork.CFHTTPMessageCreateResponse(
            None, 200, "Okidoki", CFNetwork.kCFHTTPVersion1_1
        )
        self.assertIsInstance(req, CFNetwork.CFHTTPMessageRef)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCreateEmpty)
        self.assertArgIsBOOL(CFNetwork.CFHTTPMessageCreateEmpty, 1)
        m = CFNetwork.CFHTTPMessageCreateEmpty(None, True)
        self.assertIsInstance(m, CFNetwork.CFHTTPMessageRef)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCreateCopy)
        m = CFNetwork.CFHTTPMessageCreateCopy(None, req)
        self.assertIsInstance(m, CFNetwork.CFHTTPMessageRef)

        self.assertResultIsBOOL(CFNetwork.CFHTTPMessageIsRequest)
        self.assertTrue(CFNetwork.CFHTTPMessageIsRequest(req) is True)
        self.assertTrue(CFNetwork.CFHTTPMessageIsRequest(resp) is False)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCopyVersion)
        v = CFNetwork.CFHTTPMessageCopyVersion(req)
        self.assertIsInstance(v, str)

        CFNetwork.CFHTTPMessageSetBody(req, b"hello world")

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCopyBody)
        b = CFNetwork.CFHTTPMessageCopyBody(req)
        self.assertIsInstance(b, (CFNetwork.NSData, memoryview, bytes))

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCopyAllHeaderFields)
        v = CFNetwork.CFHTTPMessageCopyAllHeaderFields(req)
        self.assertIsInstance(v, CFNetwork.CFDictionaryRef)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCopyHeaderFieldValue)
        v = CFNetwork.CFHTTPMessageCopyHeaderFieldValue(req, "X-Python")
        self.assertTrue(v is None)

        CFNetwork.CFHTTPMessageSetHeaderFieldValue(req, "X-Python", "Rocks")
        v = CFNetwork.CFHTTPMessageCopyHeaderFieldValue(req, "X-Python")
        self.assertEqual(v, "Rocks")

        self.assertResultIsBOOL(CFNetwork.CFHTTPMessageAppendBytes)
        self.assertArgHasType(CFNetwork.CFHTTPMessageAppendBytes, 1, b"n^v")
        self.assertArgSizeInArg(CFNetwork.CFHTTPMessageAppendBytes, 1, 2)

        v = CFNetwork.CFHTTPMessageAppendBytes(req, b"hello world", 11)
        self.assertTrue(v is True)

        self.assertResultIsBOOL(CFNetwork.CFHTTPMessageIsHeaderComplete)
        v = CFNetwork.CFHTTPMessageIsHeaderComplete(req)
        self.assertTrue(v is False or v is True)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCopySerializedMessage)
        v = CFNetwork.CFHTTPMessageCopySerializedMessage(resp)
        self.assertIsInstance(v, CFNetwork.CFDataRef)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCopyRequestURL)
        v = CFNetwork.CFHTTPMessageCopyRequestURL(req)
        self.assertIsInstance(v, CFNetwork.CFURLRef)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCopyRequestMethod)
        v = CFNetwork.CFHTTPMessageCopyRequestMethod(req)
        self.assertIsInstance(v, str)

        self.assertResultIsBOOL(CFNetwork.CFHTTPMessageAddAuthentication)
        self.assertArgIsBOOL(CFNetwork.CFHTTPMessageAddAuthentication, 5)
        v = CFNetwork.CFHTTPMessageAddAuthentication(
            req,
            resp,
            "ronald",
            "secret",
            CFNetwork.kCFHTTPAuthenticationSchemeBasic,
            False,
        )
        self.assertIsInstance(v, bool)

        v = CFNetwork.CFHTTPMessageGetResponseStatusCode(resp)
        self.assertIsInstance(v, int)

        self.assertResultIsCFRetained(CFNetwork.CFHTTPMessageCopyResponseStatusLine)
        v = CFNetwork.CFHTTPMessageCopyResponseStatusLine(resp)
        self.assertIsInstance(v, str)



"""
Python mapping for the CFNetwork framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""

import os
import sys

import CFNetwork._manual
import CoreFoundation
import objc
from CFNetwork import _metadata


def CFSocketStreamSOCKSGetError(err):
    return err.error & 0xFFFF


def CFSocketStreamSOCKSGetErrorSubdomain(err):
    return (err.error >> 16) & 0xFFFF


frameworkPath = "/System/Library/Frameworks/CFNetwork.framework"
if not os.path.exists(frameworkPath):
    frameworkPath = "/System/Library/Frameworks/CoreServices.framework"


sys.modules["CFNetwork"] = mod = objc.ObjCLazyModule(
    "CFNetwork",
    "com.apple.CFNetwork",
    objc.pathForFramework(frameworkPath),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "objc": objc,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "CFSocketStreamSOCKSGetError": CFSocketStreamSOCKSGetError,
        "CFSocketStreamSOCKSGetErrorSubdomain": CFSocketStreamSOCKSGetErrorSubdomain,
    },
    (CoreFoundation,),
)


for nm in dir(CFNetwork._manual):
    setattr(mod, nm, getattr(CFNetwork._manual, nm))


del sys.modules["CFNetwork._metadata"]



"""
PyObjC wrappers for the framework "CFNetwork", part of "CoreServices" on
macOS.

The CFNetwork framework provides a library of abstractions for networking
protocols. The most interesting bits for Python programmers are the
API's for working with proxy autoconfiguration and the API's for networking
diagnostics.

These wrappers don't include documentation, please check Apple's documentation
for information on how to use this framework and PyObjC's documentation
for general tips and tricks regarding the translation between Python
and (Objective-)C frameworks
"""
from pyobjc_setup import Extension, setup

VERSION = "8.5"

setup(
    name="pyobjc-framework-CFNetwork",
    description="Wrappers for the framework CFNetwork on macOS",
    packages=["CFNetwork"],
    ext_modules=[
        Extension(
            "CFNetwork._manual",
            ["Modules/_manual.m"],
            extra_link_args=["-framework", "CoreServices"],
            py_limited_api=True,
        )
    ],
    version=VERSION,
    install_requires=["pyobjc-core>=" + VERSION, "pyobjc-framework-Cocoa>=" + VERSION],
    long_description=__doc__,
    options={"bdist_wheel": {"py_limited_api": "cp36"}},
)



import CFNetwork
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestCFNetwork(TestCase):
    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertIsInstance(CFNetwork.kCFErrorDomainCFNetwork, str)
        self.assertIsInstance(CFNetwork.kCFErrorDomainWinSock, str)
        self.assertIsInstance(CFNetwork.kCFGetAddrInfoFailureKey, str)
        self.assertIsInstance(CFNetwork.kCFSOCKSStatusCodeKey, str)
        self.assertIsInstance(CFNetwork.kCFSOCKSVersionKey, str)
        self.assertIsInstance(CFNetwork.kCFSOCKSNegotiationMethodKey, str)
        self.assertIsInstance(CFNetwork.kCFDNSServiceFailureKey, str)
        self.assertIsInstance(CFNetwork.kCFFTPStatusCodeKey, str)
        self.assertIsInstance(CFNetwork.kCFURLErrorFailingURLErrorKey, str)
        self.assertIsInstance(CFNetwork.kCFURLErrorFailingURLStringErrorKey, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(CFNetwork.kCFErrorPACFileAuth, 309)
        self.assertEqual(CFNetwork.kCFErrorHTTPSProxyConnectionFailure, 310)

        self.assertEqual(CFNetwork.kCFURLErrorUnknown, -998)
        self.assertEqual(CFNetwork.kCFURLErrorCancelled, -999)
        self.assertEqual(CFNetwork.kCFURLErrorBadURL, -1000)
        self.assertEqual(CFNetwork.kCFURLErrorTimedOut, -1001)
        self.assertEqual(CFNetwork.kCFURLErrorUnsupportedURL, -1002)
        self.assertEqual(CFNetwork.kCFURLErrorCannotFindHost, -1003)
        self.assertEqual(CFNetwork.kCFURLErrorCannotConnectToHost, -1004)
        self.assertEqual(CFNetwork.kCFURLErrorNetworkConnectionLost, -1005)
        self.assertEqual(CFNetwork.kCFURLErrorDNSLookupFailed, -1006)
        self.assertEqual(CFNetwork.kCFURLErrorHTTPTooManyRedirects, -1007)
        self.assertEqual(CFNetwork.kCFURLErrorResourceUnavailable, -1008)
        self.assertEqual(CFNetwork.kCFURLErrorNotConnectedToInternet, -1009)
        self.assertEqual(CFNetwork.kCFURLErrorRedirectToNonExistentLocation, -1010)
        self.assertEqual(CFNetwork.kCFURLErrorBadServerResponse, -1011)
        self.assertEqual(CFNetwork.kCFURLErrorUserCancelledAuthentication, -1012)
        self.assertEqual(CFNetwork.kCFURLErrorUserAuthenticationRequired, -1013)
        self.assertEqual(CFNetwork.kCFURLErrorZeroByteResource, -1014)
        self.assertEqual(CFNetwork.kCFURLErrorCannotDecodeRawData, -1015)
        self.assertEqual(CFNetwork.kCFURLErrorCannotDecodeContentData, -1016)
        self.assertEqual(CFNetwork.kCFURLErrorCannotParseResponse, -1017)
        self.assertEqual(CFNetwork.kCFURLErrorInternationalRoamingOff, -1018)
        self.assertEqual(CFNetwork.kCFURLErrorCallIsActive, -1019)
        self.assertEqual(CFNetwork.kCFURLErrorDataNotAllowed, -1020)
        self.assertEqual(CFNetwork.kCFURLErrorRequestBodyStreamExhausted, -1021)
        self.assertEqual(CFNetwork.kCFURLErrorFileDoesNotExist, -1100)
        self.assertEqual(CFNetwork.kCFURLErrorFileIsDirectory, -1101)
        self.assertEqual(CFNetwork.kCFURLErrorNoPermissionsToReadFile, -1102)
        self.assertEqual(CFNetwork.kCFURLErrorDataLengthExceedsMaximum, -1103)
        self.assertEqual(CFNetwork.kCFURLErrorSecureConnectionFailed, -1200)
        self.assertEqual(CFNetwork.kCFURLErrorServerCertificateHasBadDate, -1201)
        self.assertEqual(CFNetwork.kCFURLErrorServerCertificateUntrusted, -1202)
        self.assertEqual(CFNetwork.kCFURLErrorServerCertificateHasUnknownRoot, -1203)
        self.assertEqual(CFNetwork.kCFURLErrorServerCertificateNotYetValid, -1204)
        self.assertEqual(CFNetwork.kCFURLErrorClientCertificateRejected, -1205)
        self.assertEqual(CFNetwork.kCFURLErrorClientCertificateRequired, -1206)
        self.assertEqual(CFNetwork.kCFURLErrorCannotLoadFromNetwork, -2000)
        self.assertEqual(CFNetwork.kCFURLErrorCannotCreateFile, -3000)
        self.assertEqual(CFNetwork.kCFURLErrorCannotOpenFile, -3001)
        self.assertEqual(CFNetwork.kCFURLErrorCannotCloseFile, -3002)
        self.assertEqual(CFNetwork.kCFURLErrorCannotWriteToFile, -3003)
        self.assertEqual(CFNetwork.kCFURLErrorCannotRemoveFile, -3004)
        self.assertEqual(CFNetwork.kCFURLErrorCannotMoveFile, -3005)
        self.assertEqual(CFNetwork.kCFURLErrorDownloadDecodingFailedMidStream, -3006)
        self.assertEqual(CFNetwork.kCFURLErrorDownloadDecodingFailedToComplete, -3007)
        self.assertEqual(CFNetwork.kCFHTTPCookieCannotParseCookieFile, -4000)

    def testConstants(self):
        self.assertEqual(CFNetwork.kCFHostErrorHostNotFound, 1)
        self.assertEqual(CFNetwork.kCFHostErrorUnknown, 2)
        self.assertEqual(CFNetwork.kCFSOCKSErrorUnknownClientVersion, 100)
        self.assertEqual(CFNetwork.kCFSOCKSErrorUnsupportedServerVersion, 101)
        self.assertEqual(CFNetwork.kCFSOCKS4ErrorRequestFailed, 110)
        self.assertEqual(CFNetwork.kCFSOCKS4ErrorIdentdFailed, 111)
        self.assertEqual(CFNetwork.kCFSOCKS4ErrorIdConflict, 112)
        self.assertEqual(CFNetwork.kCFSOCKS4ErrorUnknownStatusCode, 113)
        self.assertEqual(CFNetwork.kCFSOCKS5ErrorBadState, 120)
        self.assertEqual(CFNetwork.kCFSOCKS5ErrorBadResponseAddr, 121)
        self.assertEqual(CFNetwork.kCFSOCKS5ErrorBadCredentials, 122)
        self.assertEqual(CFNetwork.kCFSOCKS5ErrorUnsupportedNegotiationMethod, 123)
        self.assertEqual(CFNetwork.kCFSOCKS5ErrorNoAcceptableMethod, 124)
        self.assertEqual(CFNetwork.kCFNetServiceErrorUnknown, -72000)
        self.assertEqual(CFNetwork.kCFNetServiceErrorCollision, -72001)
        self.assertEqual(CFNetwork.kCFNetServiceErrorNotFound, -72002)
        self.assertEqual(CFNetwork.kCFNetServiceErrorInProgress, -72003)
        self.assertEqual(CFNetwork.kCFNetServiceErrorBadArgument, -72004)
        self.assertEqual(CFNetwork.kCFNetServiceErrorCancel, -72005)
        self.assertEqual(CFNetwork.kCFNetServiceErrorInvalid, -72006)
        self.assertEqual(CFNetwork.kCFNetServiceErrorTimeout, -72007)
        self.assertEqual(CFNetwork.kCFNetServiceErrorDNSServiceFailure, -73000)
        self.assertEqual(CFNetwork.kCFFTPErrorUnexpectedStatusCode, 200)
        self.assertEqual(CFNetwork.kCFErrorHTTPAuthenticationTypeUnsupported, 300)
        self.assertEqual(CFNetwork.kCFErrorHTTPBadCredentials, 301)
        self.assertEqual(CFNetwork.kCFErrorHTTPConnectionLost, 302)
        self.assertEqual(CFNetwork.kCFErrorHTTPParseFailure, 303)
        self.assertEqual(CFNetwork.kCFErrorHTTPRedirectionLoopDetected, 304)
        self.assertEqual(CFNetwork.kCFErrorHTTPBadURL, 305)
        self.assertEqual(CFNetwork.kCFErrorHTTPProxyConnectionFailure, 306)
        self.assertEqual(CFNetwork.kCFErrorHTTPBadProxyCredentials, 307)
        self.assertEqual(CFNetwork.kCFErrorPACFileError, 308)

        self.assertEqual(
            CFNetwork.kCFURLErrorBackgroundSessionInUseByAnotherProcess, -996
        )
        self.assertEqual(CFNetwork.kCFURLErrorBackgroundSessionWasDisconnected, -997)
        self.assertEqual(
            CFNetwork.kCFURLErrorAppTransportSecurityRequiresSecureConnection, -1022
        )
        self.assertEqual(CFNetwork.kCFURLErrorFileOutsideSafeArea, -1104)

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(
            CFNetwork.kCFStreamErrorHTTPSProxyFailureUnexpectedResponseToCONNECTMethod,
            311,
        )



import CFNetwork
import objc
from PyObjCTools.TestSupport import TestCase, expectedFailure


class TestCFNetwork(TestCase):
    @expectedFailure
    def testTypes(self):
        self.assertIsCFType(CFNetwork.CFNetDiagnosticRef)

    def testConstants(self):
        self.assertEqual(CFNetwork.kCFNetDiagnosticNoErr, 0)
        self.assertEqual(CFNetwork.kCFNetDiagnosticErr, -66560)
        self.assertEqual(CFNetwork.kCFNetDiagnosticConnectionUp, -66559)
        self.assertEqual(CFNetwork.kCFNetDiagnosticConnectionIndeterminate, -66558)
        self.assertEqual(CFNetwork.kCFNetDiagnosticConnectionDown, -66557)

    def testFuncdtions(self):
        self.assertResultIsCFRetained(CFNetwork.CFNetDiagnosticCreateWithStreams)

        host = CFNetwork.CFHostCreateWithName(None, "www.apple.com")
        rd, wr = CFNetwork.CFStreamCreatePairWithSocketToCFHost(
            None, host, 80, None, None
        )
        self.assertIsInstance(rd, CFNetwork.CFReadStreamRef)
        self.assertIsInstance(wr, CFNetwork.CFWriteStreamRef)

        ref = CFNetwork.CFNetDiagnosticCreateWithStreams(None, rd, wr)
        self.assertIsInstance(ref, objc.objc_object)  # CFNetDiagnosticRef)

        self.assertResultIsCFRetained(CFNetwork.CFNetDiagnosticCreateWithURL)
        ref = CFNetwork.CFNetDiagnosticCreateWithURL(
            None, CFNetwork.CFURLCreateWithString(None, "http://www.apple.com/", None)
        )
        self.assertIsInstance(ref, objc.objc_object)  # CFNetDiagnosticRef)

        CFNetwork.CFNetDiagnosticSetName(ref, "hello world")

        sts = CFNetwork.CFNetDiagnosticDiagnoseProblemInteractively(ref)
        self.assertIsInstance(sts, int)

        self.assertArgIsOut(CFNetwork.CFNetDiagnosticCopyNetworkStatusPassively, 1)
        sts, descr = CFNetwork.CFNetDiagnosticCopyNetworkStatusPassively(ref, None)
        self.assertIsInstance(sts, int)
        self.assertIsInstance(descr, str)
